@@ -1,12 +1,12 @@
 import logging
 
 from pymobiledevice3.lockdown_service_provider import LockdownServiceProvider
-from pymobiledevice3.service_connection import LockdownServiceConnection
+from pymobiledevice3.service_connection import ServiceConnection
 
 
 class LockdownService:
     def __init__(self, lockdown: LockdownServiceProvider, service_name: str, is_developer_service=False,
-                 service: LockdownServiceConnection = None, include_escrow_bag: bool = False):
+                 service: ServiceConnection = None, include_escrow_bag: bool = False):
         """
         :param lockdown: server provider
         :param service_name: wrapped service name - will attempt
@@ -27,8 +27,14 @@ class LockdownService:
     def __enter__(self):
         return self
 
+    async def __aenter__(self) -> 'LockdownService':
+        return self
+
     def __exit__(self, exc_type, exc_val, exc_tb):
         self.close()
 
+    async def __aexit__(self, exc_type, exc_val, exc_tb):
+        await self.service.aio_close()
+
     def close(self) -> None:
         self.service.close()
